#!/usr/bin/env python
#
# Unit tests for all the basic CBD blocks, discrete-time CBD.

import unittest

from CBD import *

NUM_DISCR_TIME_STEPS = 5

class BasicCBDTestCase(unittest.TestCase):
   def setUp(self):
     self.CBD = CBD("CBD_for_block_under_test")

   def _run(self, num_steps=1):
     from CBDsimulator import *
     myCBD = self.CBD
     simulator = CBDsimulator(myCBD)
     for i in range(num_steps):
        simulator.step()
     myCBD.dump()
     myCBD.dumpSignals()

   def _getSignal(self, blockname):
     foundBlocks = [ block for block in self.CBD.getBlocks() if block.getBlockName() == blockname ]
     numFoundBlocks = len(foundBlocks)
     if numFoundBlocks == 1:
       return foundBlocks[0].getSignal()
     else:
       raise Exception(str(numFoundBlocks) + " blocks with name " + blockname + " found.\nExpected a single block.")
      
   def assertEquals(self, result, oracle, msg=None):
     unittest.TestCase.assertEquals(self, result, oracle, msg)

   def testConstantBlock(self):
     self.CBD.addBlock(ConstantBlock(block_name="c1", value=3.3))

     self._run(NUM_DISCR_TIME_STEPS)
     self.assertEquals(self._getSignal("c1"), [3.3] * NUM_DISCR_TIME_STEPS)

   def testNegatorBlockPos(self):
     self.CBD.addBlock(ConstantBlock(block_name="c1", value=6.0))
     self.CBD.addBlock(NegatorBlock(block_name="n"))
     self.CBD.addConnection("c1", "n")

     self._run(NUM_DISCR_TIME_STEPS)
     self.assertEquals(self._getSignal("n"), [-6.0] * NUM_DISCR_TIME_STEPS)

   def testNegatorBlockNeg(self):
     self.CBD.addBlock(ConstantBlock(block_name="c1", value=-6.0))
     self.CBD.addBlock(NegatorBlock(block_name="n"))
     self.CBD.addConnection("c1", "n")

     self._run(NUM_DISCR_TIME_STEPS)
     self.assertEquals(self._getSignal("n"), [6.0] * NUM_DISCR_TIME_STEPS)

   #### TO IMPLEMENT: your own tescases
   #  Start your method name with "test", just like the three methods above
   #  Be complete

def suite():
     """Returns a test suite containing all the test cases in this module."""
     suite = unittest.makeSuite(BasicCBDTestCase)
     return unittest.TestSuite(suite)

if __name__ == '__main__':
     # When this module is executed from the command-line, run all its tests
     try:
        unittest.main(verbosity=2)
     except SystemExit as inst:
        if inst.args[0] is True: # raised by sys.exit(True) when tests failed
            pass
